// File.cpp: implementation of the CFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "File.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CFile::CFile()
{
	m_data = NULL;
	m_datasize = 0;
	m_bDeallocate = false;
}

CFile::CFile(std::string name,std::string filename,std::string content,BYTE* data,unsigned long datasize)
{
	assert( data != NULL );
	assert( datasize != 0 );

	m_name = name;
	m_filename = filename.substr( filename.rfind("\\") + 1 );
	m_content = content;
	m_data = data;
	m_datasize = datasize;
	m_bDeallocate = true;
}

CFile::~CFile()
{
	if( m_data != NULL && m_bDeallocate )
		delete[] m_data;
}

/*///////////////////////////////////////////////////////////////
 /                           GetData                            /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Copys the data of the file into the specified /
 /					buffer.										/
 / Parameters  :  lpData - Buffer to receive file data.         /
 /                count  - Size of lpData                       /
 / Return Value:  int - number of bytes copied.                 /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
int CFile::GetData(BYTE *lpData, size_t count)
{
	assert( lpData != NULL );
	assert( count != 0 );
	assert( m_data != NULL );
	assert( m_datasize != NULL );

	//// copy to size of caller's buffer or our buffer, which ever is smaller
	DWORD size = __min(m_datasize,count);
	memcpy( lpData, m_data, size );
	return size;
}

/*///////////////////////////////////////////////////////////////
 /                             Save                             /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Saves the data in our buffer to a file.       /
 / Parameters  :  filename - file to save to.                   /
 / Return Value:  bool - was data sucessfully saved?            /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CFile::Save(std::string filename)
{
	assert( m_data != NULL );
	assert( m_datasize != NULL );
	assert( filename.length() > 0 );
	
	/// make sure we create using the std::ios flags, not global ios flags (included in iostream.h or fstream.h)
	std::ofstream out( filename.c_str(), std::ios::binary | std::ios::out );

	/// file couldn't be opened
	if( out.fail() )
		return false;

	/// write out our buffer
	out.write( reinterpret_cast<char*>(m_data), m_datasize );

	out.close();

	return true;
}

/*///////////////////////////////////////////////////////////////
 /                              =                               /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Overload = operators is needed to that more   /
 /					than one class does not try to deallocate   /
 /					the data pointer.							/
 / Parameters  :  pFile -                                       /
 / Return Value:  CFile& - this                                 /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
CFile& CFile::operator =(const CFile &pFile)
{
	this->m_bDeallocate = false;
	this->m_content = pFile.m_content;
	this->m_data = pFile.m_data;
	this->m_datasize = pFile.m_datasize;
	this->m_filename = pFile.m_filename;
	this->m_name = pFile.m_name;
	return *this;
}

std::string CFile::GetContentType() const
{
	return m_content;
}

std::string CFile::GetClientFilename() const
{
	return m_filename;
}

std::string CFile::GetFieldName() const
{
	return m_name;
}

const BYTE* CFile::GetBuffer()
{
	return m_data;
}

DWORD CFile::GetBufferSize() const
{
	return m_datasize;
}