// MultipartForm.cpp: implementation of the CMultipartForm class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MultipartForm.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMultipartForm::CMultipartForm(EXTENSION_CONTROL_BLOCK *pECB) : CWebForm( pECB )
{
}

CMultipartForm::~CMultipartForm()
{
	/// delete all CFile objects
	m_fileit = m_files.begin();
	for( ;m_fileit != m_files.end();m_fileit++ )
		delete *m_fileit;
}

/*///////////////////////////////////////////////////////////////
 /                            Parse                             /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Parses the form data		                    /
 / Parameters  :  None                                          /
 / Return Value:  None                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
void CMultipartForm::Parse()
{
	/// if buffer is empty, we have no work to do
	if( m_buffer.IsEmpty() )
		return;

	/// line buffer
	char buf[2048];
	/// content header
	char szHeader[64];

	/// Buffer position is incremented by Append
	m_buffer.Seek( 0 );

	/// get the first line (header)
	if( !m_buffer.GetLine( szHeader, 64 ) )
		return;

	/// Seek back to start of buffer
	m_buffer.Seek( 0 );

	/// Loop through all lines in the file
	while( m_buffer.GetLine( buf,2048 )  )
	{
		/// start processing when we see a header.
		/// there should not be any empty loops.
		if( strncmp( buf, szHeader, strlen(szHeader) ) == 0 )
		{	
			/// read first line after header
			if( !m_buffer.GetLine( buf, 2048 ) )
				return;

			/// Content-Disposition: form-data; name="Name" or
			/// Content-Disposition: form-data; name="filedata"; filename="C:\..."
			std::string sBuf(buf); /// line buffer
			std::string sName; /// field name
			std::string sFile; /// file name
			std::string sContent; /// content type of file
			bool bFile = false; /// is this a file or form data

			/// find start and end of name field
			int nNamePos = sBuf.find("name=\"") + 6;
			int nNameEnd = sBuf.find("\"",nNamePos);

			assert( nNamePos > 0 );
			assert( nNameEnd > nNamePos );
			/// extract name
			sName = sBuf.substr( nNamePos, nNameEnd - nNamePos );

			/// find file field, if possible
			int nFilePos = sBuf.find( "filename=\"" ) + 10;
			int nFileEnd = sBuf.find( "\"", nFilePos );

			if( nFilePos > 10 ) // found
			{
				/// extract file name (client)
				sFile = sBuf.substr( nFilePos, nFileEnd - nFilePos );
				/// next line has content type
				/// Content-Type: image/pjpeg
				m_buffer.GetLine( buf, 2048 );
				sContent = (char*)(buf + 14);
				bFile = true;
			}
			else
				sFile = "";

			// next line will be blank.
			m_buffer.GetLine( buf, 2048 );

			/// our data starts here, we must now find the end
			DWORD startpos = m_buffer.GetPos();

			/// we found the end of the data, calculate its pos
			DWORD dataend = startpos + m_buffer.FindNext( szHeader, startpos + 1 );

			/// allocate buffer
			int nDataLen = dataend - startpos;
			
			if( nDataLen <= 0 )
				continue; /// no data here

			/// is a file
			if( bFile )
			{
				/// this shouldn't happen, but just double check.
				/// if the file name is blank, there can't be any data
				if( sFile == "" )
					continue;

				/// allocate buffer for file data
				BYTE* pbValue = new BYTE[nDataLen];
				memset(pbValue,0,nDataLen);
				/// extract file data from our buffer
				m_buffer.Copy( pbValue, nDataLen );

				//// pFile is deleted in destructor
				/// pbValue is deleted in CFile destructor
				CFile* pFile = new CFile(sName,sFile,sContent,pbValue,nDataLen);
				m_files.push_back( pFile );

				/// store file path too
				m_varmap[sName] = sFile;
			}
			else
			{
				/// this is form data, allocate buffer for it
				char* szValue = new char[nDataLen];
				// Get vale from buffer
				m_buffer.Copy( szValue, nDataLen-1 );
				/// store in var map
				m_varmap[sName] = std::string(szValue);
				delete[] szValue;
			}
		} // if( strncmp( buf, szHeader, strlen(szHeader) ) == 0 )
	} // while( m_buffer.GetLine( buf,2048 )  )
}

/*///////////////////////////////////////////////////////////////
 /                         GetFirstFile                         /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Gets a copy of the first file in the vector   /
 / Parameters  :  pFile - File object to receive the copy       /
 / Return Value:  bool - true if there are any files in the vector /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CMultipartForm::GetFirstFile(CFile *pFile)
{
	/// there is no first file if the structure is empty
	if( m_files.empty() )
		return false;

	// move to first file.
	m_fileit = m_files.begin();
	/// Copy the file data
	*pFile = **m_fileit;
	return true;
}

/*///////////////////////////////////////////////////////////////
 /                         GetNextFile                          /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Gets the next file object from the vector		/
 / Parameters  :  pFile - File object to receive the copy       /
 / Return Value:  bool - true if there was a file in the vector /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CMultipartForm::GetNextFile(CFile *pFile)
{
	/// they have asked for a file out of range
	if( m_files.end() <= m_fileit )
		return false;

	/// get the next file
	m_fileit++;
	/// copy the file structure
	*pFile = **m_fileit;
	return true;
}