// QueryString.cpp: implementation of the CQueryString class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "QueryString.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CQueryString::CQueryString(EXTENSION_CONTROL_BLOCK *pECB)
{
	assert( pECB != NULL );
	m_pECB = pECB;
}

CQueryString::~CQueryString()
{
}

/*///////////////////////////////////////////////////////////////
 /                           GetValue                           /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  retrieves the value of a specified key	    /
 / Parameters  :  key - the key whose value to retrieve         /
 / Return Value:  std::string                                   /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
std::string CQueryString::GetValue(std::string key)
{
	return m_varmap[key];
}

/*///////////////////////////////////////////////////////////////
 /                          GetValueI                           /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  retrieves the value of a specified key	    /
 / Parameters  :  key - the key whose value to retrieve         /
 / Return Value:  int                                           /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
int CQueryString::GetValueI(std::string key)
{
	return atoi(m_varmap[key].c_str());
}

/*///////////////////////////////////////////////////////////////
 /                          GetValueL                           /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  retrieves the value of a specified key	    /
 / Parameters  :  key - the key whose value to retrieve         /
 / Return Value:  long                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
long CQueryString::GetValueL(std::string key)
{
	return atol(m_varmap[key].c_str());
}

/*///////////////////////////////////////////////////////////////
 /                          GetValueD                           /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  retrieves the value of a specified key	    /
 / Parameters  :  key - the key whose value to retrieve         /
 / Return Value:  std::string                                   /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
double CQueryString::GetValueD(std::string key)
{
	return atof(m_varmap[key].c_str());
}

/*///////////////////////////////////////////////////////////////
 /                           IsEmpty                            /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Determines weather the varmap is empty		/
 / Parameters  :  None                                          /
 / Return Value:  bool                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CQueryString::IsEmpty()
{
	return m_varmap.size() == 0;
}

/*///////////////////////////////////////////////////////////////
 /                          operator[]                          /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  retrieves the value of a specified key		 /
 / Parameters  :  key - the key whose value to retrieve         /
 / Return Value:  std::string                                   /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
std::string CQueryString::operator[](std::string key)
{
	return GetValue(key);
}

/*///////////////////////////////////////////////////////////////
 /                            Parse                             /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Parses the form/query string into the varmap  /
 / Parameters  :  None						                    /
 / Return Value:  None                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
void CQueryString::Parse()
{
	Parse( m_pECB->lpszQueryString );
}

/*///////////////////////////////////////////////////////////////
 /                            Parse                             /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Parses the form/query string into the varmap  /
 / Parameters  :  szQuery - string to parse                     /
 / Return Value:  None                                          /
 / Remarks     :  Used to allow CWebForm to use the same code.  /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
void CQueryString::Parse(const char *szQuery)
{
	m_varmap.clear();

	/// Get Length of query string
	DWORD dwLen = strlen( szQuery );
	if( dwLen <= 0 )
		return; // can't process empty string

	/// create a copy of the query string so we can modify it
	char* szQueryString = new char[dwLen+1];
	unsigned int nVars = 0; /// how many query string variables are there
	strcpy( szQueryString, szQuery );

	// replace all + with _ and & with nulls
	for( int i = 0;i < dwLen-1;i++ )
	{
		/// + is escape char for space
		if( szQueryString[i] == '+' )
			szQueryString[i] = ' ';
		/// and is used to append two variables
		else if( szQueryString[i] == '&' )
		{
			szQueryString[i] = '\0';
			nVars++;
		}
	}
	
	char* pszKey = szQueryString;
	/// loop through variables
	for( i = 0;i < nVars+1;i++ )
	{
		/// replace escape characters with thier actual characters
		EscapeToAscii( pszKey );

		/// find the equals
		char* pszValue = strchr( pszKey, '=' ) + 1;
		strtok( pszKey, "=" );
		std::string key(pszKey);
		std::string value;

		/// equals wasn't found, make value = key for qs' like isapi.dll?page1
		if( pszValue < pszKey )
		{
			value = key;
			pszKey = strchr( pszKey, 0 ) + 1;
		}
		else
		{
			value = pszValue;
			pszKey = strchr( pszValue, 0 ) + 1;
		}

		/// add to var map
		m_varmap[key] = value;
	}

	delete[] szQueryString;
}

/*///////////////////////////////////////////////////////////////
 /                        EscapeToAscii                         /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Convers all escape characters (%??) into thier/
 /				   ASCII values.								/
 / Parameters  :  lpEscape - Pointer to a string to modify.     /
 / Return Value:  None                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
void CQueryString::EscapeToAscii(char* pString)
{
	int j = 0;
	for( int i = 0; j < strlen(pString); i++, j++ )
	{
		if( (pString[i] = pString[j]) == '%' )
		{
			pString[i] = HexToAscii( &pString[j+1] );
			j+=2;
		}
	}

	pString[i] = '\0';
}

/*///////////////////////////////////////////////////////////////
 /                          HexToAscii                          /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Converts a two-digit hex code into it's ASCII /
 /					value.										/
 / Parameters  :  lpString - two-digit hex code                 /
 / Return Value:  char                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
char CQueryString::HexToAscii(const char *pString) const
{
	char ch;
	ch = (pString[0] >= 'A' ? ( (pString[0] & 0xDF) - 'A' ) + 10 : (pString[0] - '0') ) * 16;
	ch += (pString[1] >= 'A' ? ( (pString[1] & 0xDF) - 'A' ) + 10 : (pString[1] - '0') );
	return ch;
}