// WebForm.cpp: implementation of the CWebForm class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "WebForm.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWebForm::CWebForm(EXTENSION_CONTROL_BLOCK *pECB) : CQueryString( pECB )
{
	assert( pECB != NULL );
	m_pECB = pECB;
	ReadFormData();
}

CWebForm::~CWebForm()
{
}

/*///////////////////////////////////////////////////////////////
 /                         ReadFormData                         /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Reads the form from the client into a buffer. /
 / Parameters  :  None                                          /
 / Return Value:  bool                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CWebForm::ReadFormData()
{
	DWORD dwTotalRead = 0; /// data we have read
	DWORD dwBufferSize = 0; /// total size of all data

	//// Get content length from server
	dwBufferSize = m_pECB->cbTotalBytes;

	/// if nothing to read, we are done
	if( dwBufferSize == 0 )
		return true;

	/// Allocate new buffer
	if( !m_buffer.Allocate( dwBufferSize ) )
		return false;

	/// Retrieve Server's buffer (first 48kb only)
	dwTotalRead = m_pECB->cbAvailable;
	if( !m_buffer.Append( m_pECB->lpbData, dwTotalRead ) )
		return false;

	/// read rest of buffer from client
	while( dwTotalRead < dwBufferSize )
	{
		DWORD dwBtyesToRead = 512;
		BYTE pbBuffer[512]; /// temporary buffer

		memset( pbBuffer, 0, 512 );

		/// when we hit the end, we can't read 512 bytes, so decrease read size
		if( (dwTotalRead + dwBtyesToRead) > dwBufferSize )
			dwBtyesToRead = dwBufferSize - dwTotalRead;

		//// read from client to temporary buffer
		if( !m_pECB->ReadClient(m_pECB->ConnID, pbBuffer, &dwBtyesToRead) )
			return false;

		/// append temp buffer to read buffer
		if( dwBtyesToRead > 0 )
			m_buffer.Append( pbBuffer, dwBtyesToRead );

		dwTotalRead += dwBtyesToRead;
	}

	return true;
}

/*///////////////////////////////////////////////////////////////
 /                            Parse                             /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Parses the form data into the vector.         /
 / Parameters  :  None                                          /
 / Return Value:  None                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
void CWebForm::Parse()
{
	/// get the buffer to parse
	DWORD dwSize = m_buffer.Size();
	char* szBuffer = new char[dwSize+1];
	memset( szBuffer, 0, dwSize+1 );
	m_buffer.ToChar( szBuffer, dwSize );

	/// Input is same as query string
	CQueryString::Parse( szBuffer );
	delete[] szBuffer;
}

/*///////////////////////////////////////////////////////////////
 /                           IsEmpty                            /
 ////////////////////////////////////////////////////////////////
 /                                                              /
 / Author      :  Michael Williamson                            /
 / Description :  Was there any form data received?		        /
 / Parameters  :  None                                          /
 / Return Value:  bool                                          /
 / Remarks     :                                                /
 /                                                              /
 ///////////////////////////////////////////////////////////////*/
bool CWebForm::IsEmpty()
{
	return m_buffer.IsEmpty();
}
